/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.net.impl;

import org.xmloperator.lambda2.net.model.Cell;
import org.xmloperator.lambda2.net.model.PentaryMultiplexor;
import org.xmloperator.lambda2.net.model.Port;
import org.xmloperator.lambda2.net.model.Stack;

/**
 * Multiplexor implementation.
 */
public final class PentaryMultiplexorImpl extends CellImpl
		implements
			PentaryMultiplexor {
	private static final boolean ASSERTION = Test.ASSERTION;

	final boolean isAbstractor;
	final PortImpl bodyPort = new PortImpl(this, 1);
	final PortImpl variablePort = new PortImpl(this, 2);
	final PortImpl duplicatorPort2 = new PortImpl(this, 3);
	final PortImpl duplicatorPort1 = new PortImpl(this, 4);
	final PortImpl duplicatorPort0 = new PortImpl(this, 5);

	public PentaryMultiplexorImpl(boolean isAbstractionMultiplexor) {
		this.isAbstractor = isAbstractionMultiplexor;
	}

	//////////////////////
	// Cell implementation
	//////////////////////

	public final int getSymbol() {
		return this.isAbstractor
				? SYMBOL_ABSTRACTOR
				: SYMBOL_APPLICATOR;
	}

	public int getSymbolArity() {
		return 5;
	}
	
	public final boolean isPentaryMultiplexor() {
		return true;
	}

	public Port getPort(int portNumber) {
		switch (portNumber) {
			case 0 :
				return this;
			case 1 :
				return this.bodyPort;
			case 2 :
				return this.variablePort;
			case 3 :
				return this.duplicatorPort2;
			case 4 :
				return this.duplicatorPort1;
			case 5 :
				return this.duplicatorPort0;
			default :
				throw new IllegalArgumentException();
		}
	}

	public void collectPorts(Stack portStack) {
		super.collectPorts(portStack);
		portStack.push(this.duplicatorPort0);
		portStack.push(this.duplicatorPort1);
		portStack.push(this.duplicatorPort2);
		portStack.push(this.variablePort);
		portStack.push(this.bodyPort);
	}

	public Cell cloneCell() {
		return new PentaryMultiplexorImpl(this.isAbstractor);
	}

	public final int interact() {
		PortImpl connectedPort = this.connectedPort;
		if (connectedPort.portNumber != 0)
			throw new IllegalStateException("Not an active pair");
		CellImpl cell = connectedPort.cell;
		switch (cell.getSymbol()) {
			case SYMBOL_WIRE :
				return cell.interact();
			case SYMBOL_ERASER :
				this.bodyPort.connectedPort.connectTo(cell);
				this.variablePort.connectedPort.connectTo(new EraserCellImpl());
				this.duplicatorPort2.connectedPort
						.connectTo(new EraserCellImpl());
				this.duplicatorPort1.connectedPort
						.connectTo(new EraserCellImpl());
				this.duplicatorPort0.connectedPort
						.connectTo(new EraserCellImpl());
				return this.isAbstractor
						? INTERACTION_ERASING_ABSTRACTOR
						: INTERACTION_ERASING_APPLICATOR;
			case SYMBOL_DUPLICATOR :
			case SYMBOL_CONSTRUCTOR :
				return this.interactWithBinaryCell((BinaryCellImpl) cell);
			case SYMBOL_ABSTRACTOR :
			case SYMBOL_APPLICATOR :
				return this.interactWithPentaryMultiplexor((PentaryMultiplexorImpl) cell);
			case SYMBOL_DECODER :
				return cell.interact();
			default:
				throw new IllegalStateException("Unknown symbol");
		}
	}

	private final int interactWithBinaryCell(BinaryCellImpl binaryCell) {
		if (!binaryCell.isDuplicator)
			throw new IllegalStateException(
					"Unsupported interaction between a constructor and a multiplexor");
		PortImpl connectedPort11 = binaryCell.port1.connectedPort;
		PortImpl connectedPort12 = binaryCell.port2.connectedPort;
		if (ASSERTION) {
			Cell cell11 = connectedPort11.cell;
			Cell cell12 = connectedPort12.cell;
			if (cell11 == binaryCell || cell11 == this
					|| cell12 == this)
				throw new IllegalStateException(
						"Illegal interconnection");
		}
		PortImpl connectedPort21 = this.bodyPort.connectedPort;
		PortImpl connectedPort22 = this.variablePort.connectedPort;
		PortImpl connectedPort23 = this.duplicatorPort2.connectedPort;
		PortImpl connectedPort24 = this.duplicatorPort1.connectedPort;
		PortImpl connectedPort25 = this.duplicatorPort0.connectedPort;
		if (ASSERTION
				&& (connectedPort23.cell == this
						|| connectedPort24.cell == this || (this.isAbstractor
						? connectedPort25 == this.bodyPort
						: connectedPort21.cell == this
								|| connectedPort22.cell == this
								|| connectedPort25.cell == this)))
			throw new IllegalStateException("Illegal interconnection");
		PentaryMultiplexorImpl cell21 = this;
		PentaryMultiplexorImpl cell22 = new PentaryMultiplexorImpl(
				this.isAbstractor);
		if (connectedPort22.cell == this) {
			if (connectedPort22 == this.bodyPort) {
				BinaryCellImpl cell15 = new BinaryCellImpl(true);
				cell21.variablePort.connectTo(cell21.bodyPort);
				cell22.variablePort.connectTo(cell22.bodyPort);
				connectedPort25.connectTo(cell15);
				cell15.port1.connectTo(cell21.duplicatorPort0);
				cell15.port2.connectTo(cell22.duplicatorPort0);
			} else if (connectedPort22 == this.duplicatorPort0) {
				BinaryCellImpl cell11 = new BinaryCellImpl(true);
				cell21.variablePort.connectTo(cell21.duplicatorPort0);
				cell22.variablePort.connectTo(cell22.duplicatorPort0);
				connectedPort21.connectTo(cell11);
				cell11.port1.connectTo(cell21.bodyPort);
				cell11.port2.connectTo(cell22.bodyPort);
			} else
				throw new IllegalStateException(
						"Illegal interconnection");
		} else {
			BinaryCellImpl cell11 = new BinaryCellImpl(true);
			BinaryCellImpl cell12 = new BinaryCellImpl(true);
			BinaryCellImpl cell15 = new BinaryCellImpl(true);
			connectedPort21.connectTo(cell11);
			connectedPort22.connectTo(cell12);
			connectedPort25.connectTo(cell15);
			cell11.port1.connectTo(cell21.bodyPort);
			cell11.port2.connectTo(cell22.bodyPort);
			cell12.port1.connectTo(cell21.variablePort);
			cell12.port2.connectTo(cell22.variablePort);
			cell15.port1.connectTo(cell21.duplicatorPort0);
			cell15.port2.connectTo(cell22.duplicatorPort0);
		}
		BinaryCellImpl cell13 = binaryCell;
		BinaryCellImpl cell14 = new BinaryCellImpl(true);
		connectedPort11.connectTo(cell21);
		connectedPort12.connectTo(cell22);
		connectedPort23.connectTo(cell13);
		connectedPort24.connectTo(cell14);
		cell13.port1.connectTo(cell21.duplicatorPort2);
		cell13.port2.connectTo(cell22.duplicatorPort2);
		cell14.port1.connectTo(cell21.duplicatorPort1);
		cell14.port2.connectTo(cell22.duplicatorPort1);
		return this.isAbstractor
				? INTERACTION_DUPLICATING_ABSTRACTOR
				: INTERACTION_DUPLICATING_APPLICATOR;
	}
	
	private final int interactWithPentaryMultiplexor(PentaryMultiplexorImpl multiplexor) {
		if (multiplexor.isAbstractor == this.isAbstractor)
			throw new IllegalStateException(
					"Unsupported interaction between two "
							+ (this.isAbstractor
									? "abstractors"
									: "applicators"));
		PentaryMultiplexorImpl abstractor = this.isAbstractor
				? this
				: multiplexor;
		PentaryMultiplexorImpl applicator = this.isAbstractor
				? multiplexor
				: this;
		PortImpl connectedPort11 = abstractor.bodyPort.connectedPort;
		PortImpl connectedPort12 = abstractor.variablePort.connectedPort;
		PortImpl connectedPort13 = abstractor.duplicatorPort2.connectedPort;
		PortImpl connectedPort14 = abstractor.duplicatorPort1.connectedPort;
		PortImpl connectedPort15 = abstractor.duplicatorPort0.connectedPort;
		PortImpl connectedPort21 = applicator.bodyPort.connectedPort;
		PortImpl connectedPort22 = applicator.variablePort.connectedPort;
		PortImpl connectedPort23 = applicator.duplicatorPort2.connectedPort;
		PortImpl connectedPort24 = applicator.duplicatorPort1.connectedPort;
		PortImpl connectedPort25 = applicator.duplicatorPort0.connectedPort;
		if (ASSERTION) {
			Cell cell21 = connectedPort21.cell;
			Cell cell22 = connectedPort22.cell;
			Cell cell23 = connectedPort23.cell;
			Cell cell24 = connectedPort24.cell;
			Cell cell25 = connectedPort25.cell;
			if (cell21 == abstractor || cell21 == applicator
					|| cell22 == abstractor || cell22 == applicator
					|| cell23 == abstractor || cell23 == applicator
					|| cell24 == abstractor || cell24 == applicator
					|| cell25 == abstractor || cell25 == applicator
					|| connectedPort13.cell == abstractor
					|| connectedPort14.cell == abstractor
					|| connectedPort15 == abstractor.bodyPort)
				throw new IllegalStateException(
						"Illegal interconnection");
		}
		if (connectedPort12.cell == abstractor) {
			if (connectedPort12 == abstractor.bodyPort) {
				connectedPort22.connectTo(connectedPort21);
				connectedPort15.connectTo(connectedPort25);
			} else if (connectedPort12 == abstractor.duplicatorPort0) {
				connectedPort22.connectTo(connectedPort25);
				connectedPort11.connectTo(connectedPort21);
			} else
				throw new IllegalStateException(
						"Illegal interconnection");
		} else {
			connectedPort11.connectTo(connectedPort21);
			connectedPort12.connectTo(connectedPort22);
			connectedPort15.connectTo(connectedPort25);
		}
		connectedPort13.connectTo(connectedPort23);
		connectedPort14.connectTo(connectedPort24);
		return INTERACTION_ABSTRACTOR_APPLICATOR;
	}
	
	/////////////////////////////
	// Multiplexor implementation
	/////////////////////////////

	public final boolean isAbstractionMultiplexor() {
		return this.isAbstractor;
	}

	public final Port getBodyPort() {
		return this.bodyPort;
	}

	public final Port getVariablePort() {
		return this.variablePort;
	}

	public final Port getDuplicatorPort0() {
		return this.duplicatorPort0;
	}

	public final Port getDuplicatorPort1() {
		return this.duplicatorPort1;
	}

	public final Port getDuplicatorPort2() {
		return this.duplicatorPort2;
	}
}