/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.impl;

import java.util.Map;
import java.util.Set;

import org.xmloperator.lambda2.term.model.Variable;

/**
 * One variable Abstraction implementation.
 */
public final class Abstraction1Impl extends Abstraction0Impl {
	final Variable boundVariable;

	public Abstraction1Impl(TermImpl body, Variable boundVariable) {
		super(body);
		if (boundVariable == null)
			throw new IllegalArgumentException();
		this.boundVariable = boundVariable;
	}

	final void addFreeVariables(Set boundVariables, Set freeVariables) {
		if (!boundVariables.add(this.boundVariable))
			throw new IllegalStateException(
					"Two abstractions for the same bound variable");
		this.body.addFreeVariables(boundVariables, freeVariables);
		if (ASSERTION && boundVariables.contains(this.boundVariable))
			throw new IllegalStateException(
					"Bound variable unused inside its abstraction");
	}

	final void toText(StringBuffer buffer, int boundVariableStackSize,
			Map fromBoundVariableToName) {
		String variableName = "x" + Integer.toString(++boundVariableStackSize);
		if (fromBoundVariableToName.put(this.boundVariable, variableName) != null)
			throw new IllegalStateException(
					"Two abstractions for the same bound variable");
		buffer.append('\\');
		buffer.append(variableName);
		buffer.append('.');
		this.body.toText(buffer, boundVariableStackSize,
				fromBoundVariableToName);
		if (ASSERTION
				&& fromBoundVariableToName.containsKey(this.boundVariable))
			throw new IllegalStateException(
					"Bound variable unused inside its abstraction");
	}

	// ///////////////////////////
	// Abstraction implementation
	// ///////////////////////////

	public int getBoundVariableCount() {
		return 1;
	}

	public Variable getBoundVariable(int index) {
		if (index != 0)
			throw new IllegalArgumentException();
		return this.boundVariable;
	}
}