/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.xmloperator.lambda2.term.model.Abstraction;
import org.xmloperator.lambda2.term.model.Application;
import org.xmloperator.lambda2.term.model.Term;
import org.xmloperator.lambda2.term.model.Variable;

/**
 * Term implementation.
 */
public abstract class TermImpl implements Term {

	/**
	 * Adds the free variables of this Term, that contains a given set of bound
	 * variables, to a given set of free variables.
	 * 
	 * @param boundVariables
	 *            a Set of bound variables
	 * @param freeVariables
	 *            the Set of free variables.
	 */
	abstract void addFreeVariables(Set boundVariables, Set freeVariables);

	/**
	 * Apend a textual representation of this Term to the given StringBuffer.
	 * 
	 * @param buffer
	 *            a StringBuffer
	 * @param boundVariableStackSize
	 *            the size of bound variable stack.
	 * @param fromBoundVariableToName
	 *            a bound variable naming.
	 */
	abstract void toText(StringBuffer buffer, int boundVariableStackSize,
			Map fromBoundVariableToName);

	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return this.toText();
	}

	//////////////////////
	// Term implementation
	//////////////////////

	public boolean isVariable() {
		return false;
	}

	public boolean isAbstraction() {
		return false;
	}

	public boolean isApplication() {
		return false;
	}

	public final Set freeVariables() {
		Set boundVariables = new HashSet();
		Set freeVariables = new HashSet();
		this.addFreeVariables(boundVariables, freeVariables);
		return freeVariables;
	}

	public final String toText() {
		StringBuffer buffer = new StringBuffer();
		Map fromVariableToName = new HashMap();
		this.toText(buffer, 0, fromVariableToName);
		return buffer.toString();
	}

	public Abstraction buildAbstraction() {
		return new Abstraction0Impl(this);
	}

	public Abstraction buildAbstraction(Variable boundVariable) {
		return new Abstraction1Impl(this, boundVariable);
	}

	public Abstraction buildAbstraction(Variable[] boundVariables) {
		switch (boundVariables.length) {
			case 0 :
				return new Abstraction0Impl(this);
			case 1 :
				return new Abstraction1Impl(this, boundVariables[0]);
			default :
				return new AbstractionNImpl(this, boundVariables);
		}
	}

	public Application buildApplication(Term argument) {
		return new Application1Impl(this, (TermImpl) argument);
	}

	public Application buildApplication(Term argument, long functionExponent) {
		return functionExponent != 1L ? new ApplicationNImpl(this,
				(TermImpl) argument, functionExponent) : new Application1Impl(
				this, (TermImpl) argument);
	}
}