/**
 * Copyright (c) 2005 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda2/).
 */
package org.xmloperator.lambda2.term.model;

import org.xmloperator.lambda2.net.model.TooMuchReductionStepException;

/**
 * Lambda Term reducer.
 */
public interface TermReducer {

	/**
	 * Returns the normal form, if it exists, of the given Term.
	 * 
	 * @param term
	 *            a Term.
	 * @return the given Term in normal form, if it exists.
	 * @throws IllegalArgumentException
	 *             if the given term has free variable(s).
	 * @throws TooMuchReductionStepException
	 *             if the given step max count is not enough.
	 */
	public Term normalForm(Term term) throws TooMuchReductionStepException;

	/**
	 * Returns the boolean value of the given Term, if it represents a boolean.
	 * 
	 * @return the boolean value of the given Term, if it represents a boolean.
	 * @throws IllegalArgumentException
	 *             if the given Term has free variable(s) or doesn't represent a
	 *             boolean value.
	 */
	public boolean booleanValue(Term booleanTerm);

	/**
	 * Returns the Church numeral value represented by the given Term if it
	 * represents one, elsewhere returns -1L.
	 * 
	 * @return the Church numeral value represented by the given Term if it
	 *         represents one, elsewhere returns -1L.
	 * @throws IllegalArgumentException
	 *             if the given term has free variable(s).
	 */
	public long numeralValue(Term numeralTerm);
}
